/** ColoredTableCellRenderer.java

 Used to colorized the automata line tables to distinguish
 different submachines, different adaptive functions, the
 rules and primitive actions being executed and so on.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/

package adaptools.vm;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;
import java.io.*;
import javax.swing.table.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;

/*

 Helps the implementation of 'colorized tables', like the
 one representing the automaton.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
class ColoredTableCellRenderer extends DefaultTableCellRenderer {

  private Hashtable Colors = new Hashtable();
  private Hashtable TextColors = new Hashtable();

  public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,boolean hasFocus, int row, int column)
  {
    Component result = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
    if(!isSelected && !hasFocus) {
      Color bg = getRowColor(row); // determine appropriate bg colour
      Color fg = getTextRowColor(row); // determine appropriate fg colour
      if (bg!=null)
      {
        setBackground(bg);
      }
      if (fg!=null)
      {
        setForeground(fg);
      }
    }
    return result;
  }

  public void setRowColor(int row, Color c)
  {
    Colors.put(new Integer(row), c);
  }

  public Color getRowColor(int row)
  {
    return (Color) Colors.get(new Integer(row));
  }

  public void setTextRowColor(int row, Color c)
  {
    TextColors.put(new Integer(row), c);
  }

  public Color getTextRowColor(int row)
  {
    return (Color) TextColors.get(new Integer(row));
  }


  public void init(int totalRows, Color back, Color fore)
  {
	  for( int i = 0 ; i < totalRows ; ++i ) {
		  Colors.put(new Integer(i), back);
		  TextColors.put(new Integer(i), fore);
      }
  }
}