/** VmdsImpl.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/

package adaptools.vm;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

/**

 Implements a simple html browser that
 shows the on-line Help (coded in the HTML file readme.html)

 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
class HelpDialog extends JDialog implements HyperlinkListener {

	private String initialURL="readme.html";
    private JEditorPane htmlPane;
    private JTextField urlField;
    private JButton homeButton;
    private JLabel urlLabel;

    Container contentPane;
	GridBagLayout gridbag;
	GridBagConstraints c;

    public HelpDialog(Frame aFrame) {
        super(aFrame, true);


        homeButton = new javax.swing.JButton();
        homeButton.setBackground(new java.awt.Color(102, 255, 102));
        homeButton.setText("HOME");
        homeButton.setPreferredSize(new java.awt.Dimension(530, 26));
        homeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                homeButtonActionPerformed(evt);
            }
        });


        urlLabel = new javax.swing.JLabel("URL: ");
        urlField = new javax.swing.JTextField(40);


        try {
	      initialURL = "file://"+(new File(initialURL)).getAbsoluteFile();
          urlField.setText(initialURL);
          htmlPane = new JEditorPane(initialURL);
          htmlPane.setEditable(false);
          htmlPane.addHyperlinkListener(this);
        } catch(IOException ioe) {
          System.err.println("Can't build HTML pane for " + initialURL+ ": " + ioe);
        }


        JScrollPane readmeScrollPane = new JScrollPane(htmlPane);
        readmeScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        readmeScrollPane.setPreferredSize(new Dimension(550, 400));


        contentPane = getContentPane();
        gridbag = new GridBagLayout();
        c = new GridBagConstraints();
        contentPane.setLayout(gridbag);


        // Dimension: width = 7, height = 7.
        setConstraints(urlLabel,         0 , 0 , 1 , 1, null);
        setConstraints(urlField,         1 , 0 , 1 , 1, null);
        setConstraints(readmeScrollPane, 0 , 1 , 2 , 1, null);
        setConstraints(homeButton,       0 , 2 , 2 , 1, null);


        setLocation(aFrame.getX()+180,aFrame.getY()+40);

        pack();
        show();

   }

   private void setConstraints( JComponent jc, int x, int y, int w, int h, Insets insets) {
        c.gridwidth = w;
        c.gridheight= h;
        c.gridx = x;
        c.gridy = y;
        if(insets != null) c.insets = insets;
        gridbag.setConstraints(jc, c);
        contentPane.add(jc);
   }

   public void hyperlinkUpdate(HyperlinkEvent event) {
     if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
       try {
         htmlPane.setPage(event.getURL());
         urlField.setText(event.getURL().toExternalForm());
       } catch(IOException ioe) {
         System.out.println("Can't follow link to "
                 + event.getURL().toExternalForm() + ": " + ioe);
       }
     }
   }


   public void homeButtonActionPerformed(ActionEvent event) {
       String url;
       if (event.getSource() == urlField)
         url = urlField.getText();
       else  // Clicked "home" button instead of entering URL
         url = initialURL;
       try {
         htmlPane.setPage(new URL(url));
         urlField.setText(url);
       } catch(IOException ioe) {
         System.out.println("Can't follow link to " + url + ": " + ioe);
       }
   }



}
