/** Kernel.java

 Runs an Structured PushDown Automata.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/

package adaptools.vm;

import java.util.List;

import adaptools.utils.ListSemanticsActions;
import adaptools.visual.VisualMachine;



/**

 Runs Adaptive Automata, calling the {@link Adapter} to
 execute adaptive functions. The java class carring the
 semantics of the machine is linked to it by the method
 {@link getSemanticObject}, using information from
 the name of the object code file. For instance, if the
 name contains "lex", this method will assign the
 adaptools.examples.ccl.Semantics class to handle semantics. The method
 {@link tryToExecute} starts the execution of
 code, in adaptools.examples.ccl.Semantics, implementing the
 semantic of some specific output symbol. You must study
 this methods, and the ones called by it, in order to learn
 how to implement another semantics handling code.


 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
public class Kernel implements Runnable {

    private Vmds vmds; // Virtual Machine data structures.
    private Adapter adapter;
    private Semantics semantics;
    private List semanticsAvailable; 
    private Rule lastRule;
    private Token lastToken = null;
    private boolean hasToken = false;
    private VisualMachine lastVisual = null;
    private boolean modified = false;
    
    //private LoadClass load = new LoadClass();

    private int contaLoop;
    
    /**
     * default builder
     * @param vmds
     */
    public Kernel(Vmds vmds) {        
		
    	this.vmds = vmds;		
		
    	//semanticsAvailable = new ArrayList();    	
    	semanticsAvailable = new ListSemanticsActions().getSemanticsAvailable();
    	
    	   	
    	// faz busca por classes com aes semanticas disponiveis
		//ListClassesSemantics(ListFilesSubs.doGetDir()+"semantics/");
		
		semantics = getSemanticObject();
		adapter = new Adapter(vmds);

		//System.out.println("Classes = "+semanticsAvailable);
		
        Thread t = new Thread(this);        
        t.start();        
    }
    
    
    /**
     * Lista o Diretrio padro procurando por classes que implementam aes Semanticas
     *
     */
    /*private void ListClassesSemantics(String directory){
    	
    	File[] files;
    	
    	// Faz chamada recursiva para todos os subdiretrios
    	files = ListFilesSubs.doFileFilterDirectoryListing(directory);
    	for(int i=0 ; i < files.length ; i++)
    		ListClassesSemantics(files[i].getAbsolutePath());
    	
    	// Recupera os arquivos com extenso class
    	files = ListFilesSubs.doFileFilterListing(directory,".*[.]class");
    	
    	
    	
    	for(int i=0 ; i< files.length ; i++ ){
    		try {
    			// recupera o caminho absoluto da classe    			
    			URI uri = files[i].toURI();
    			String nameClass = uri.getPath(); // transformei em uri para funcionar no Windows e Linux
    			
    			
    			nameClass = nameClass.replaceAll(ListFilesSubs.doGetDir(),"");
    			
    			nameClass = nameClass.replaceAll("[.]class","");
    			
    			nameClass = nameClass.replaceAll("[/]",".");
    			nameClass = nameClass.replaceFirst("semantics[.]","");
    	
    			System.out.println(ListFilesSubs.doGetDir()+" classe = "+nameClass);
    			
    			Class testSemantic = load.loadClass(nameClass);

				//Class testSemantic = Class.forName(nameClass);
				
				// verifica se a classe  uma implemtao de Semantics
				// se for adiciona a lista de aes Semanticas Disponveis				 
				if(isInterface(testSemantic.getInterfaces(), Semantics.class))
					semanticsAvailable.add(nameClass);
					
			} catch (ClassNotFoundException e) {				
				e.printStackTrace();
			}
    	}
    }
    
    
    private boolean isInterface(Class[] interfaces, Class interf){
    	for(int i=0 ; i < interfaces.length ; i++)
    		if(interfaces[i].getName().equals(interf.getName()))
    			return true;
    		
    	return false; 
    }*/

    public Semantics getSemanticObject() {
    	Semantics semanticImpl = null;
    	
    	for(int i = 0 ; i < semanticsAvailable.size() ; i++){
    		// se for a ao semantica procurada
    		if(semanticsAvailable.get(i).toString().equals( vmds.getSemanticAction() )) {
    			try {
					Class testSemantic = Class.forName(semanticsAvailable.get(i).toString());
					
					try {
						semanticImpl = (Semantics)testSemantic.newInstance();
					} catch (InstantiationException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				    semanticImpl.init(vmds);
				    
				    return semanticImpl;
					
				} catch (ClassNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} 
    		}
    	}
    	
    	System.out.println("Default Semantics Loaded");
	     
    	semanticImpl = new adaptools.vm.DefaultSemantics();
	    semanticImpl.init(vmds);
	     
	    return semanticImpl;
    	
    	
		/*if(  vmds.getObjectFileName().indexOf("sintatico") != -1 ) {
		     System.out.println("CC Semantics Loaded");
		     
		     /*semanticImpl = new adaptools.examples.cc.Semantics();
		     semanticImpl.init(vmds);
		     
	    }*/
    }

    public void run() {
		extractCode();
//		mountLookAheadTable();
		if(vmds.getInputMachine() != null) {
            waitInputMachineToBeLoaded();
	    }
        if(vmds.getOutputMachine() != null) {
            wakeupOutputMachine();
        }
		stepByStep();
		vmds.kernelWillDie();
    }



    /** 
     * 	This method get the object code from the vmds and convert it to
     *  a more eficient representation (not implemented yet).
     *  @deprecated
     */
    private void extractCode() {
    }

/*    private void mountLookAheadTable() {

		lookAheadTable = new Hashtable();

		int rule = 0;

		while( vmds.validRule(rule) ) {
		  if( ! adapter.isFunction(rule) && ! adapter.isRemovedRule(rule) ) {
			 if( rule.isEmptyJump() ) {
		     }
	      }
		  ++rule;
	    }
    }*/



    public void stepByStep() {
		Rule rule = null;
		int dest;
		modified = false;

		vmds.initDS();
		vmds.showCurrentToken();
        lastRule = null;
		rule = vmds.getCode().getFirstRule();
		vmds.setCurState(rule.getOriginInt()); // Set the current state to the origin of the first rule.

        if(vmds.isAnimate()) lastVisual = vmds.getCodeTable().takeFirstPicture();
        if(vmds.isAnimateSingle()) lastVisual.setResizable(false);

		while( (rule = getNextRule()) != null ) {

            boolean removed = false;

			if(rule.hasBeforeAction()) {
				if(adapter.execute(rule.beforeAction(),rule.getHeader())==true) {
                   modified = true;
                }
				if(vmds.isAnimate()) {
				    VisualMachine temp = lastVisual;
				    lastVisual.formatRemovedRules();
				    if(vmds.isAnimateMulti()) {
						lastVisual = vmds.getCodeTable().takePicture();
			    	    temp.stopLayouting();
					}
				    else {
				   
				    	lastVisual.addInsertedRules();
				    }
			    }
                removed = rule.isRemoved(); // Must be saved because the rule can be definitely removed.
		        if(vmds.isRemoveDefinitely()) vmds.removeDefinitely();
		    }
			
			if(!removed) {
    			if( (dest = getDestValue(rule)) == Vmds.STACK_UNDERFLOW ) {
    				break;
    		    } else {
    			    vmds.setCurState(dest);
    		    }
    			if(rule.hasOutput()) {
    				vmds.appendTransOutput(rule.getOutput());
    				vmds.appendOutput(tryToExecute(rule));
    		    }
    			if(rule.isSubMachineCall()) {
    				vmds.push(rule.getPushInt());
    		    }
    			if(!rule.isEmptyJump()) {
    		        vmds.readToken();
    		    }
    			if(rule.hasAfterAction()) {
					if(adapter.execute(rule.afterAction(),rule.getHeader())==true) { 
                        modified = true; 
                    }
				    if(vmds.isAnimate()) {
				       VisualMachine temp = lastVisual;
				       lastVisual.formatRemovedRules();
				       if(vmds.isAnimateMulti()) {
						   lastVisual = vmds.getCodeTable().takePicture();
						   temp.stopLayouting();
					   }
				       else lastVisual.addInsertedRules();
			        }
		            if(vmds.isRemoveDefinitely()) vmds.removeDefinitely();
		        }
		    }
	    }

		if(accepted()==true) vmds.appendTransOutput("[Accepted]");
		else vmds.appendTransOutput("[Rejected]");
		if(vmds.getOutputMachine() != null) setLastToken(null);
		if(vmds.getInputMachine() != null) {
			vmds.getInputMachine().getKernel().tokenRead();
			vmds.getInputMachine().setRunning(false);
	    }
		if(vmds.isAnimateMulti()) lastVisual.stopLayouting();
		vmds.setRunning(false);
		vmds.executionFinished();
    }

    // Not optimized yet (too slow).
	private Rule getNextRule() {
		
		boolean found = false;
		// If there are no transition for the current input symbol and
		// there is a rule with meta input 'other', use it.
		Rule otherJump = null;

		// If there are no better choice, use any available empty rule.
		Rule anyEmptyJump = null;

		Rule rule = vmds.getCode().getFirstRule();

		while( rule.isValid() ) {
		  if(!rule.isFunction() && !rule.isRemoved() &&
		      vmds.getCurState()==rule.getOriginInt() ) {
     			if( vmds.curToken() != null &&
     			    rule.reads(vmds.curToken())) {
					found = true;
					break;
			    } else {
					if( rule.isOther() && vmds.curToken() != null ) otherJump = rule;
					if( rule.isEmptyJump() ) {
                        if(vmds.curToken() != null ) {
                            contaLoop = 0;
							if(aheadIsOK(rule)) {
								found = true;
                                break;
						    } else {
								anyEmptyJump = rule;
						    }
					    } else {
							if(!rule.isErrorJump()) {
							    if(aheadIsOK(rule)) {
								  found = true;
                                  break;
						        } else {
								  anyEmptyJump = rule;
               			        }
						    }
					    }
				    }
			    }
	      }
		  rule = vmds.getCode().getRule(rule.getIndex()+1);
	    }

        if(!found && (anyEmptyJump != null || otherJump != null )) {
			found = true;
			if(otherJump != null) rule = otherJump; // otherJump has higher priority than empty
			else {
               rule = anyEmptyJump;
            }
	    }

	    if(lastRule != null) vmds.unsetRule(lastRule);
	    lastRule = rule;

	    if(found ) {
		   vmds.setRule(rule);
		   waitSomeTime();
		   return(rule);
	    } else {
	       return(null);
	    }
	    
    }

    private boolean aheadIsOK(Rule rule) {
		boolean ret=false;
		Rule next = vmds.getCode().getFirstRule();
              
        if( rule.isSubMachineReturn()) return(true);

		while( next.isValid() ) {

           
            if(!next.isFunction() && !next.isRemoved()
		       && next.getOriginInt() == rule.getDestinationInt()) {
            	
                if(rule.hasAfterAction() || rule.hasBeforeAction()) {
					ret = true;
					break;
			    }
			    if(next.isEmptyJump() && contaLoop < 7) {
           
                    ++contaLoop;                    
					ret = aheadIsOK(next);
                    --contaLoop;
					if(ret == true) break;
			    }
                if( (vmds.curToken() != null && next.reads(vmds.curToken())) ||
                    (vmds.curToken() == null && next.isSubMachineReturn())
                    )
                {
                  ret=true;
                  break;
			    }
	      }
		  next = vmds.getCode().getRule(next.getIndex()+1);

	    }
	    return(ret);
    }

    /** Returns true if the subjacent mechanism was modified during execution */
    public boolean wasModified() {
       return(modified);
    }

    
    public void setModified(boolean value) {
       modified = value;
    }

    
    // BE CAREFUL !!!! A POP IS MADE IF DEST IS "POP"
    private int getDestValue(Rule rule) {
		if(rule.isSubMachineReturn()) {
			return( vmds.pop() );
	    } else {
			return( rule.getDestinationInt());
	    }
    }


    private String tryToExecute(Rule rule) {
		Token token = semantics.execute(rule.getOutput());
		if(token != null) {
			if(vmds.getOutputMachine() != null) setLastToken(token);
			return token.label;
	    }
		else return "";
    }

    private synchronized void setLastToken(Token token) {
		 if(hasToken == false) {
			lastToken = token;
			hasToken = true;
			if(vmds.getOutputMachine().getKernel() != null)
			   vmds.getOutputMachine().getKernel().tokenOK();
            try {
     	       vmds.setSleeping(true);
               wait();
            } catch (InterruptedException e) {}
	     }
    }

    public synchronized Token getLastToken() {
		return(lastToken);
    }

    // Called by consumer to signal that the current token was read
    public synchronized void tokenOK() {
		vmds.setSleeping(false);
	    notifyAll();
    }

    // Called by consumer to signal that the current token was read
    public synchronized void waitToken() {
        try {
     	   vmds.setSleeping(true);
           wait();
        } catch (InterruptedException e) {}
    }

    // Called by consumer to signal that the current token was read
    public synchronized void tokenRead() {
		hasToken = false;
   	    vmds.setSleeping(false);
	    notifyAll();
    }

    public void waitSomeTime() {
		if(vmds.getStepSpeed() == 0) {
          waitUser();
	    } else {
          try {
              // Wait 500milliseconds before continuing
        	  /*TODO: Why I have to wait... I dont Undestand yet..
        	   *
        	   *     GAUCHO, see this code.... I am goin to debug the adaptools code
        	   *     if you understand this.. plese talk to me.
        	   *
        	   *     Denys G Santos 
        	   */  
              Thread.sleep((int) ((Vmds.MAX_SPEED-vmds.getStepSpeed())/20.0)*1000);
          } catch (InterruptedException e) {
             System.out.println(e);
          }
	    }
    }

    public synchronized void waitUser() {
        try {
            wait();
         } catch (InterruptedException e) {}
    }

    public synchronized void userClicked() {
		notifyAll();
    }



    public synchronized void waitInputMachineToBeLoaded() {
        try {
           wait();
        } catch (InterruptedException e) {}
    }

    public synchronized void wakeupOutputMachine() {
		notifyAll();
    }


    private boolean accepted() {
		if( vmds.allInputRead() &&
		    vmds.isStackEmpty() &&
		    vmds.currentInFinals() ) {
			return(true);
	    } else {
	        return(false);
	    }

    }

    public Adapter getAdapter() {
		return adapter;
    }

    public static void main(String args[]) {

       System.out.println("Must be executed from adaptools.vm.vmds");

    }


	/**
	 * @return Returns the hasToken.
	 */
	public boolean isHasToken() {
		return hasToken;
	}


	/**
	 * @param hasToken The hasToken to set.
	 */
	public void setHasToken(boolean hasToken) {
		this.hasToken = hasToken;
	}


}
