/** ObjectCodeTable.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/

package adaptools.vm;

import java.awt.Color;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;

import adaptools.visual.VisualMachine;

/**

 Add some specific features to the table representing the
 virtual machine object code. Like a pop-up menu and header
 tooltips.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
class ObjectCodeTable extends JTable implements	ActionListener, MouseListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 7169155811577293912L;
	
	private	JPopupMenu	popupMenu;
	private int popupRow = 1;
	private int lastX = 10;
	private int lastY = 10;
	private Vmds vmds;
	public static String animationHeader = null;
	public static int animationPosX = 0;
	public static int animationPosY = 0;

	private boolean changed = false;

	private static int number = 1;

	public ObjectCodeTable(Vmds vmds)
	{
		this.vmds = vmds;
		JMenuItem menuShowGraph = new JMenuItem( "Sub-Machine Graph" );
		JMenuItem menuNewLine = new JMenuItem( "New Rule" );
		JMenuItem menuRemoveLine = new JMenuItem( "Remove Rule" );
        popupMenu = new JPopupMenu( "Options" );
		popupMenu.add( menuShowGraph );
		popupMenu.add( menuNewLine );
		popupMenu.add( menuRemoveLine );

		//topPanel.add( popupMenu );

		// Action and mouse listener support
		//enableEvents( AWTEvent.MOUSE_EVENT_MASK );

        menuRemoveLine.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuRemoveLineActionPerformed(evt);
                  }
        });


        menuShowGraph.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuShowGraphActionPerformed(evt);
                  }
        });

        menuNewLine.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuNewLineActionPerformed(evt);
                  }
        });


	}



    public void setToolTips(String [] msgs) {

        JTableHeader header = getTableHeader();

        TableHeaderToolTips tips = new TableHeaderToolTips();

        for(int c=0; c < msgs.length;++c) {
		    tips.setToolTip(getColumnModel().getColumn(c),msgs[c]);
	    }

        header.addMouseListener(this);
        header.addMouseMotionListener(tips);
        header.setForeground(Color.white);
    }

    public VisualMachine takeFirstPicture() {

	    Rule rule = vmds.getCode().getFirstRule();
        animationHeader = rule.getHeader();

        if(animationHeader != null) {
            animationPosX = 200;
            animationPosY = 100;
            return(new VisualMachine(vmds,animationHeader,animationPosX, animationPosY));
	    }
	    return(null);
    }

    public VisualMachine takePicture() {

		if(animationHeader == null) {
			System.out.println("PROBLEM!!! - call takeFirstPicture FIRST");
	    }

        animationPosX += 20;
        animationPosY += 25;
        return (new VisualMachine(vmds,animationHeader,animationPosX, animationPosY));
    }

    public void tableChanged(TableModelEvent e) {
		super.tableChanged(e);
		changed = true;
    }
/*
    public void scrollToRow(int row) { 
        Rectangle result = getCellRect(row, -1, true);
        Insets i = getInsets();
        result.x = i.left;
        result.width = getWidth() - i.left - i.right;

        scrollRectToVisible(result);
    }*/


    public void scrollToRow(int row, int viewHeight, int barValue) { 
        Rectangle result = getCellRect(row, -1, true);
        Insets i = getInsets();
        result.x = i.left;
        result.width = getWidth() - i.left - i.right;

        boolean forcedPaint = false;

        int modulo = result.y % viewHeight;
        if((result.y >= barValue+viewHeight) || ((result.y+result.height) < barValue)) {
           forcedPaint = true; 
        }
        scrollRectToVisible(result);
        if(forcedPaint) repaint();
    }

    public void resetChanged() {
		changed = false;
    }

    public boolean getChanged() {
		return changed;
    }

    private void menuRemoveLineActionPerformed(java.awt.event.ActionEvent event) {

        vmds.getCode().removeRow(popupRow);
        vmds.colorizeAllRules();

    }



    private void menuNewLineActionPerformed(java.awt.event.ActionEvent event) {

        Rule rowRule = vmds.getCode().getRule(popupRow);
        Rule rule = new Rule(rowRule.getNuplaClone(),popupRow);
        rule.setInput(rowRule.getInput());
        rule.setOutput(rowRule.getOutput());
        rule.setOrigin(rowRule.getOrigin());
        rule.setDestination(rowRule.getDestination());
        rule.setAdaptiveAction("nop");
        rule.setPush("nop");
        vmds.getCode().insertRow(popupRow,rule.getNuplaClone());
        vmds.colorizeAllRules();

    }



    private void menuShowGraphActionPerformed(java.awt.event.ActionEvent event) {

        String header = vmds.getCode().getRule(popupRow).getHeader();
//        System.out.println("header = "+header);

        if(header != null) {
            new VisualMachine(vmds,header,lastX+100, lastY+20);
	    }
    }


	public void processMouseEvent( MouseEvent event )
	{

		if( event.isPopupTrigger() )
		{
            popupRow = rowAtPoint(event.getPoint());
            popupMenu.setLightWeightPopupEnabled(false);
			popupMenu.show( event.getComponent(), event.getX(), event.getY()%this.getPreferredSize().height );
			lastX = event.getX();
			lastY = event.getY();
		} else {
		    super.processMouseEvent( event );
        }
	}


	public void actionPerformed( ActionEvent event )
	{
		// Add action handling code here
System.out.println("Entrou em action Performed");
		System.out.println( event );
	}


    public void mouseClicked(MouseEvent e) {

        boolean ascending = true;
        JTableHeader header = getTableHeader();

		if(e.getModifiers()==InputEvent.BUTTON3_MASK) ascending = false;

        sortAllRowsBy(header.columnAtPoint(e.getPoint()),ascending);
        vmds.colorizeAllRules();

    }

    public void mouseEntered(MouseEvent e) {};
    public void mouseExited(MouseEvent e) {};
    public void mousePressed(MouseEvent e)  {};
    public void mouseReleased(MouseEvent e) {};

    public void sortAllRowsBy(int colIndex, boolean ascending) {
        setAutoCreateColumnsFromModel(false);
//		System.out.println("Chamou sort col = "+colIndex+" ascen = "+ascending);
        Vector data = ((DefaultTableModel)getModel()).getDataVector();
        Collections.sort(data, new ColumnSorter(colIndex, ascending));
        ((DefaultTableModel)getModel()).fireTableStructureChanged();
        setAutoCreateColumnsFromModel(true);
    }

    // This comparator is used to sort vectors of data
    public class ColumnSorter implements Comparator {
        int colIndex;
        boolean ascending;
        ColumnSorter(int colIndex, boolean ascending) {
            this.colIndex = colIndex;
            this.ascending = ascending;
        }
        public int compare(Object a, Object b) {
            Vector v1 = (Vector)a;
            Vector v2 = (Vector)b;
            Object o1 = v1.get(colIndex);
            Object o2 = v2.get(colIndex);

            // Treat empty strains like nulls
            if (o1 instanceof String && ((String)o1).length() == 0) {
                o1 = null;
            }
            if (o2 instanceof String && ((String)o2).length() == 0) {
                o2 = null;
            }

            // Sort nulls so they appear last, regardless
            // of sort order
            if (o1 == null && o2 == null) {
                return 0;
            } else if (o1 == null) {
                return 1;
            } else if (o2 == null) {
                return -1;
            } else if (o1 instanceof Comparable) {
                if (ascending) {
                    return ((Comparable)o1).compareTo(o2);
                } else {
                    return ((Comparable)o2).compareTo(o1);
                }
            } else {
                if (ascending) {
                    return o1.toString().compareTo(o2.toString());
                } else {
                    return o2.toString().compareTo(o1.toString());
                }
            }
        }
    }

}

