/** Rule.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/


package adaptools.vm;

import java.util.StringTokenizer;
import java.util.Vector;

/**

 Represents both rules from the subjacent mechanism (automaton transitions) and
 elementary adaptive actions.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/

/**
 * @Deprecated
 */
public class Rule {

	Vector nupla;
	int index;

	/**
	 * 
	 * @param 8-upla
	 * @param index
	 */
    public Rule(Vector nupla, int index) {
		this.nupla = nupla;
		this.index = index;
    }

    /**
     *  get Element data
     * @param i
     * @return String
     */
    public String getElement(int i) {
		return( (String) nupla.elementAt(i));
    }

    /**
     * get origin
     * @return String
     */
    public String getOrigin() {
		return( (String) nupla.elementAt(Rules.ORIG_COL));
    }

    /**
     * get origem int value
     * @return int
     */
    public int getOriginInt() {
		try {
		   return(Integer.parseInt(getOrigin()));
        } catch (NumberFormatException e) {
			return(Rules.INVALID_VALUE);
	    }
    }

    /**
     * get destination
     * @return String
     */
    public String getDestination() {
		return((String) nupla.elementAt(Rules.DEST_COL));
    }

    /**
     * get Destinationint
     * @return int
     */
    public int getDestinationInt() {
		try {
		   return(Integer.parseInt(getDestination()));
        } catch (NumberFormatException e) {
			return(Rules.INVALID_VALUE);
	    }
    }

    /**
     * get Header
     * @return String
     */
    public String getHeader() {
		return((String) nupla.elementAt(Rules.HEAD_COL));
    }

    /**
     * get Time
     * @return String
     */
    public String getTime() {
		return((String) nupla.elementAt(Rules.TIME_COL));
    }

    /**
     * get adaptative Action
     * @return string
     */
    public String getAdaptiveAction() {
		return((String) nupla.elementAt(Rules.ADAP_COL));
    }

    /**
     * get push
     * @return push
     */
    public String getPush() {
		return((String) nupla.elementAt(Rules.PUSH_COL));
    }

    /**
     * get push int value
     * @return int
     */
    public int getPushInt() {
	    try {
     		return Integer.parseInt(getPush());
        }
        catch (NumberFormatException e)
        {
        	return(Rules.INVALID_VALUE);
        }
    }

    /**
     * get input value
     * @return String
     */
    public String getInput() {
		return (String) nupla.elementAt(Rules.INPU_COL);
    }

    /**
     * get output value
     * @return String
     */
    public String getOutput() {
		return (String) nupla.elementAt(Rules.OUTP_COL);
    }

    /**
     * get intdex
     * @return int
     */
    public int getIndex() {
		return(index);
    }

    /**
     * get nupla clone
     * @return Vector
     */
    public Vector getNuplaClone() {
		return( (Vector) nupla.clone());
    }

    /**
     * get Final state
     * @return String
     */
    public String getFinalState() {
        String state = "";
		if(isSubMachineReturn()) state = getOrigin();
		else state = getDestination();
	    return state;
    }

    /**
     * is Final
     * @return
     */
    public boolean hasFinal() {
       if( !isFunction() && getPush().equals("fin")) return true;
       else return false;
    }

    @SuppressWarnings("unchecked")
	public void setTime(String time) {
        this.nupla.setElementAt(time,Rules.TIME_COL);
    }

    @SuppressWarnings("unchecked")
	public void setOrigin(String time) {
        nupla.setElementAt(time,Rules.ORIG_COL);
    }

    @SuppressWarnings("unchecked")
	public void setDestination(String time) {
        nupla.setElementAt(time,Rules.DEST_COL);
    }


    @SuppressWarnings("unchecked")
	public void setInput(String time) {
        nupla.setElementAt(time,Rules.INPU_COL);
    }

    @SuppressWarnings("unchecked")
	public void setOutput(String time) {
        nupla.setElementAt(time,Rules.OUTP_COL);
    }

    @SuppressWarnings("unchecked")
	public void setAdaptiveAction(String time) {
        nupla.setElementAt(time,Rules.ADAP_COL);
    }

    @SuppressWarnings("unchecked")
	public void setPush(String time) {
        nupla.setElementAt(time,Rules.PUSH_COL);
    }


    @SuppressWarnings("unchecked")
	public void setRemoved() {
	    nupla.setElementAt( "#"+getHeader(),Rules.HEAD_COL);
    }

    // Return true if the rule reads the input symbol
    public boolean reads(Token token) {
//		System.out.println("in read - token = "+token.label+" input = "+getInput());
        if(token.label.equals(getInput())) return true;
        if(inputIsRange() && isInRange(getInput(),token.label)) return true;
        if(inputIsLetter() && (isInRange("a..z",token.label) ||
                               isInRange("A..Z",token.label))) return true;
        if(inputIsDigit() && isInRange("0..9",token.label)) return true;
        if(inputIsSpecial() && isSpecial(token.label)) return true;
        if(inputIsSpace() && isSpace(token.label)) return true;
        return false;
	}

	private boolean inputIsRange() {
//		System.out.println("in range - indexOf = "+getInput().indexOf(".."));
		return getInput().length() == 4 && getInput().indexOf("..") >= 0;
    }

    private boolean isInRange(String range, String str) {
		if(str.length() != 1) return false;
		char ch = str.charAt(0);
        char first = range.charAt(0);
        char last = range.charAt(3);
//        System.out.println("Is "+ch+" in range: "+first+".."+last);
        if(ch >= first && ch <= last) return true;
        return false;
    }

    private boolean isSpace(String str) {
		if(str.length() != 1) return false;
		char ch = str.charAt(0);
	    if( ch == ' ' || ch == '\n' || ch == '\r' || ch == '\t') return true;
        return false;
    }

    private boolean inputIsLetter() {
		return getInput().equals("letter");
    }

    private boolean inputIsDigit() {
		return getInput().equals("digit");
    }

    private boolean inputIsSpecial() {
		return getInput().equals("special");
    }
    
    private boolean inputIsSpace() {
		return getInput().equals("spc");
    }

    public boolean isOther() {
		return getInput().equals("other");
    }


    private boolean isSpecial(String str) {
		if(str.length() != 1) return false;
		char ch = str.charAt(0);
        if(ch >= 'a' && ch <= 'z' || ch >= 'A' && ch <= 'Z' || ch >= '0' &&
           ch <= '9') return false;
        return true;

    }

    public boolean hasOutput() {
		String value = getOutput();
		if( value.equals("nop") ) {
			return( false );
	    } else {
			return( true );
	    }
    }

    public boolean isSubMachineCall() {
		String value = getPush();
		if( value.equals("nop") || value.equals("fin" ) ) {
			return( false );
	    } else {
			return( true );
	    }
    }

    public boolean isRemoved() {
		return isHeaderMarked( getHeader());
    }

    public static boolean isHeaderMarked(String str) {
		return  str.substring(0,1).equals("#");
    }
   
    /**
     * One Rule is valid when this has the origin value not 
     * <code>NULL</code> end diferent <code>""</code>
     * @return <code>TRUE</code> if this rule is valid 
     */
    public boolean isValid() {
		String value = getOrigin();
		if( value != null &&  !value.equals("") ) return true;
		else return false;
    }

    public boolean isEmptyJump() {
		return( getInput().equals("eps"));
    }

    /*
     *It is more one "GAMBIARRA"
     *
     *TODO: go to remove this. the tools do not have to adeque to the automata type, is only
     *have to adeque to theory
     *
     */
    public boolean isErrorJump() {
		return( getAdaptiveAction().equals("ERR"));
    }

    public boolean isSubMachineReturn() {
        return( getDestination().equals("pop") );
    }

    public boolean isDestinationFinal() {
		if( isSubMachineReturn() ) return false;
        else return( getPush().equals("fin") );
    }

    public String toString() {
		return("< "+getIndex()+" - "+getHeader()+"("+getOrigin()+","+getInput()+","+getDestination()+","+getPush()+","+getOutput()+")"+" ["+getAdaptiveAction()+"] >");
    }


    /*
     *************************************************************** 
     ***********  A D A P T I V E    M E C H A N I S M   ***********
     ***************************************************************
     */


    /**
     * check if this is a adaptive Function. (The Rule is Function when it's a primitive) 
     * @return <code>TRUE</code> if it's Function
     */
    public boolean isFunction()  {
        return isPrimitive(getHeader());
    }

    /**
     * chec if the headerValue is of adaptive action
     * @param headerValue
     * @return
     */
    public static boolean isFunction(String headerValue)  {
        return isPrimitive(headerValue);
    }

    /**
     * check if the parameter is member of this adaptative functions.
     * (this is used only when this rule is an member of one adaptative
     * function) 
     * @param function
     * @return <code>true</code> if this parameter is member of this adaptative function 
     */
    public boolean isMemberOf(String function)  {
        return(function.equals(getHeader().substring(1)));
    }

    /**
     * verify if this rule has any action (before or after)
     * @return <code>true</code> if its hava any adaptative action
     */
	public boolean hasAction() {
		if(hasAfterAction() || hasBeforeAction()) return(true);
		else return(false);
    }

	/**
	 * Verify if this rule has after adaptative action
	 * @return <code>Ttrue</code> if this has after action
	 */
    public boolean hasAfterAction() {
		String action = getAdaptiveAction();

		if(action.indexOf(".") >= 0
		   && action.indexOf(".") < (action.length()-1))
		   return(true);
		else return(false);
    }

    /**
     * Check if this rule has an action that has to be executed 
     * before the token walk 
     * @return <code>true</code> if this is has Before Action
     */
    public boolean hasBeforeAction() {
		String action = getAdaptiveAction();

		if(action.indexOf(".") >= 1)
		   return(true);
		else return(false);
    }

	public String afterAction() {
		String action = getAdaptiveAction();
		String before = action.substring(action.indexOf(".")+1);
		return(before);
    }

	public String beforeAction() {
		String action = getAdaptiveAction();
		/* pega  a strinn coorespondente a que vem antes do token */
		String after = action.substring(0,action.indexOf("."));
		return(after);
    }


    public static String actionName(String action) {
		String name = action;
		if(name.indexOf("(") >= 0)
		   name = action.substring(0,name.indexOf("("));
		return(name);

    }

    // If action header has parameters return string containing parameters
    //    separated by comma
    // Else return null
    public static String getParameters(String action) {
		if(action.indexOf("(") >= 0)
		   return(action.substring(action.indexOf("(")+1));
		else return null;
    }

    public static Vector parseActionField(String actionField) {
        return(parseList(actionField));
    }

   

    public static Vector actionParameters(String action) {
        // Extract the name of the function and the first open parenthesis.
	    String parsComma = getParameters(action);
        return(parseList(parsComma));
    }

    @SuppressWarnings("unchecked")
	private static Vector parseList(String list) {
	    if(list != null) {

	       Vector pars = new Vector();
           StringTokenizer tokens = new StringTokenizer(list,"(,)");

           while(tokens.hasMoreTokens()) {
              String value = tokens.nextToken();
              pars.addElement(value);
	       }
		   return(pars);
	    }
	    else return null;
    }

    public static boolean hasInternalVar(String actionField) {
		if( actionField.indexOf("?") >= 2 ) return true;
        else return false;
    }

    public static boolean hasGenVar(String actionField) {
		if( actionField.indexOf("*") >= 2 ) return true;
        else return false;
    }


    /**
     * check if the string contain the query var
     * @param string
     * @return <code>true</code> if it have a query var.
     */
    public static boolean isQueryVar(String str) {
    	/*
    	 *TODO: All query would be seted by a global static declaration
    	 *
    	 *  Ex: 
    	 *  	public static String QUERY_VAR = "?";
    	 *		public static String INSERTION_VAR = "+";
    	 *		public static String REMOTION_VAR = "-";
    	 *		public static String GENERATE_VAR = "*";
    	 *		public static String DELETE_VAR = "#";
    	 *		public static String EPSILON_VAR = "eps";
    	 *		public static String NO_OPERATIONAL_VAR = "nop";
    	 *		public static String FINAL_VAR = "fin";
    	 *		public static String POP_VAR = "pop";
    	 */
		return  str.substring(0,1).equals("?");
    }

    public static boolean isGenVar(String str) {
		return  str.substring(0,1).equals("*");
    }

    /**
     * The Rule is a primitive if  it's query, insertion or remotion
     * tipe.
     * @param String
     * @return return <code>TRUE</code> if it's primitive
     */
    public static boolean isPrimitive(String str) {
		return  isQuery(str) ||
		        isInsertion(str) ||
		        isRemotion(str);
    }

    /**
     * Verify if the parameter is member of elementar <em>Query</em> action
     * @param <code>String</code>
     * @return <code>TRUE</code> is member of elementar query action
     */
    public static boolean isQuery(String str) {
		return  str.substring(0,1).equals("?");
    }

    /**
     * Verify if the parameter is member of elementar <em>Insertion</em> action
     * @param <code>String</code>
     * @return <code>TRUE</code> is member of elementar insertion action
     */
    public static boolean isInsertion(String str) {
		return  str.substring(0,1).equals("+");
    }

    /**
     * Verify if the parameter is member of elementar <em>Remotion</em> action
     * @param <code>String</code>
     * @return <code>TRUE</code> is member of elementar remotion action
     */
    public static boolean isRemotion(String str) {
		return  str.substring(0,1).equals("-");
    }

    /**
     * Verify if this rule is a <em>query</em> adaptive action
     * @return <code>TRUE</code> if this is an elementar query action
     */
    public boolean isQuery() {
		return  isQuery(getHeader());
    }

    /**
     * Verify if this rule is a <em>Insertion</em> adaptive action
     * @return <code>TRUE</code> if this is an elementar insertion action 
     */
    public boolean isInsertion() {
		return  isInsertion(getHeader());
    }

    /**
     * Verify if this rule is a <em>Remotion</em> adaptive action
     * @return <code>TRUE</code> if this is an elementar remotion action
     */
    public boolean isRemotion() {
		return isRemotion(getHeader());
    }


}
