/** Rules.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/


package adaptools.vm;

import java.util.Vector;

import javax.swing.table.DefaultTableModel;

/**
    A set of rules

    @see rule

    @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
public class Rules extends DefaultTableModel {

    /**
	 * Serial version UID
	 */
	private static final long serialVersionUID = 1747987754979693053L;
	
	public static int TOTAL_COLUMNS = 8;
    public static int INVALID_VALUE = -1;
    public static int TIME_COL = 0; // Header column (Names of submachines and adaptive functions)
    public static int HEAD_COL = 1; // Header column (Names of submachines and adaptive functions)
    public static int ORIG_COL = 2; // Origin column number
    public static int INPU_COL = 3; // Input column number
    public static int DEST_COL = 4; // Destination column number
    public static int PUSH_COL = 5; // Push column number
    public static int OUTP_COL = 6; // Output column number
    public static int ADAP_COL = 7; // Adaptive Action

    public Rules(int y, int x) {
       super(y,x);
    }

    // Not optimized yet (too slow).
	public Rule getRule(int index) {
	    return(new Rule((Vector)getDataVector().elementAt(index),index));
    }

    public String machineContaining( String state ) {

		int i = 0;
		Rule rule;

		while( (rule = getRule(i)).isValid() ) {
		  if( !rule.isFunction() && !rule.isRemoved() && state.equals(rule.getOrigin())) {
		        return( rule.getHeader() );
		  }
		  ++i;
	    }
	    return("Unkown");
    }

    /**
     *  Not optimized yet (too slow).
     *  
     *  pega a primeira regra se esta regra for valida
     *  
     */
	public Rule getFirstRule() {
	   int i = 0;
	   Rule rule = getRule(i);
	   while( rule.isValid() ) {
		  if( !rule.isFunction() && !rule.isRemoved() ) {
			  return(rule);
	      }
		  rule = getRule(rule.getIndex()+1);
	    }
	    System.out.println("ERROR - No valid rule found");
	    return(null);
	}

    // Not optimized yet (too slow).
	public Rule getFirstIncludingRemovedRule() {
	   int i = 0;
	   Rule rule = getRule(i);
	   while( rule.isValid() ) {
		  if( !rule.isFunction() ) {
			  return(rule);
	      }
		  rule = getRule(rule.getIndex()+1);
	    }
	    System.out.println("ERROR - No valid rule found");
	    return(null);
	}


    // Trick to enable repainting of row (for instance, to mark current rule)
    public void refreshRule(int row) {
  		for(int i = 0; i < TOTAL_COLUMNS; ++i ) {
		   setValueAt( (String) getValueAt(row,i),row,i);
	    }
    }


}
